searchData={"items":[{"type":"module","title":"Extrace","doc":"`Extrace` provides functions for tracing events in a safe\nmanner for single Erlang virtual machine, currently for function\ncalls only.\n\nFunctionality includes:\n\n- Nicer to use interface (arguably) than `:dbg` or trace BIFs.\n- Protection against dumb decisions (matching all calls on a node\n  being traced, for example)\n- Adding safe guards in terms of absolute trace count or\n  rate-limiting\n- Nicer formatting than default traces","ref":"Extrace.html"},{"type":"module","title":"Tracing Elixir and Erlang Code - Extrace","doc":"The Erlang Trace BIFs allow to trace any Elixir and Erlang code at\nall. They work in two parts: pid specifications, and trace patterns.\n\nPid specifications let you decide which processes to target. They\ncan be specific pids, `all` pids, `existing` pids, or `new` pids\n(those not spawned at the time of the function call).\n\nThe trace patterns represent functions. Functions can be specified\nin two parts: specifying the modules, functions, and arguments, and\nthen with Erlang match specifications to add constraints to\narguments (see `calls/3` for details).\n\nWhat defines whether you get traced or not is the intersection of\nboth:\n\n    .       _,--------,_      _,--------,_\n         ,-'            `-,,-'            `-,\n      ,-'              ,-'  '-,              `-,\n     |   Matching    -'        '-   Matching    |\n     |     Pids     |  Getting   |    Trace     |\n     |              |   Traced   |  Patterns    |\n     |               -,        ,-               |\n      '-,              '-,  ,-'              ,-'\n         '-,_          _,-''-,_          _,-'\n             '--------'        '--------'\n\nIf either the pid specification excludes a process or a trace\npattern excludes a given call, no trace will be received.","ref":"Extrace.html#module-tracing-elixir-and-erlang-code"},{"type":"module","title":"Example Session - Extrace","doc":"First let's trace the `:queue.new` functions in any process:\n\n    > Extrace.calls({:queue, :new, :_}, 1)\n    1\n    13:14:34.086078 <0.44.0> :queue.new\n    Recon tracer rate limit tripped.\n\nThe limit was set to `1` trace message at most, and `Extrace`\nlet us know when that limit was reached.\n\nLet's instead look for all the `:queue.in/2` calls, to see what it\nis we're inserting in queues:\n\n    > Extrace.calls({:queue, :in, 2}, 1)\n    1\n    13:14:55.365157 <0.44.0> :queue.in(a, {[], []})\n    Recon tracer rate limit tripped.\n\nIn order to see the content we want, we should change the trace\npatterns to use a `fn` that matches on all arguments in a list\n(`_`) and returns `:return`. This last part will generate a second\ntrace for each call that includes the return value:\n\n    > Extrace.calls({:queue, :in, fn(_) -> :return end}, 3)\n    1\n\n    13:15:27.655132 <0.44.0> :queue.in(:a, {[], []})\n\n    13:15:27.655467 <0.44.0> :queue.in/2 --> {[:a], []}\n\n    13:15:27.757921 <0.44.0> :queue.in(:a, {[], []})\n    Recon tracer rate limit tripped.\n\nMatching on argument lists can be done in a more complex manner:\n\n    > Extrace.calls(\n    ...>   {:queue, :_,\n    ...>    fn([a, _]) when is_list(a); is_integer(a) andalso a > 1 -> :return end}\n    ...>   {10, 100}\n    ...> )\n    32\n\n    13:24:21.324309 <0.38.0> :queue.in(3, {[], []})\n\n    13:24:21.371473 <0.38.0> :queue.in/2 --> {[3], []}\n\n    13:25:14.694865 <0.53.0> :queue.split(4, {[10, 9, 8, 7], [1, 2, 3, 4, 5, 6]})\n\n    13:25:14.695194 <0.53.0> :queue.split/2 --> {{[4, 3, 2], [1]}, {[10, 9, 8, 7],[5, 6]}}\n\n    > Extrace.clear\n    :ok\n\nNote that in the pattern above, no specific function (`_`) was\nmatched against. Instead, the `fn` used restricted functions to\nthose having two arguments, the first of which is either a list or\nan integer greater than `1`.\n\nThe limit was also set using `{10, 100}` instead of an integer,\nmaking the rate-limiting at 10 messages per 100 milliseconds,\ninstead of an absolute value.\n\nAny tracing can be manually interrupted by calling\n`Extrace.clear/0`, or killing the shell process.\n\nBe aware that extremely broad patterns with lax rate-limiting (or\nvery high absolute limits) may impact your node's stability in ways\n`Extrace` cannot easily help you with.\n\nIn doubt, start with the most restrictive tracing possible, with low\nlimits, and progressively increase your scope.\n\nSee `calls/3` for more details and tracing possibilities.","ref":"Extrace.html#module-example-session"},{"type":"module","title":"Structure - Extrace","doc":"This library is production-safe due to taking the following\nstructure for tracing:\n\n```\n[IO/Group leader] <---------------------,\n  |                                     |\n[shell] ---> [tracer process] ----> [formatter]\n```\n\nThe tracer process receives trace messages from the node, and\nenforces limits in absolute terms or trace rates, before forwarding\nthe messages to the formatter. This is done so the tracer can do as\nlittle work as possible and never block while building up a large\nmailbox.\n\nThe tracer process is linked to the shell, and the formatter to the\ntracer process. The formatter also traps exits to be able to handle\nall received trace messages until the tracer termination, but will\nthen shut down as soon as possible.\n\nIn case the operator is tracing from a remote shell which gets\ndisconnected, the links between the shell and the tracer should make\nit so tracing is automatically turned off once you disconnect.\n\nIf sending output to the Group Leader is not desired, you may specify\na different `pid()` via the option `:io_server` in the `calls/3`\nfunction. For instance to write the traces to a file you can do\nsomething like\n\n    > {:ok, dev} = File.open(\"/tmp/trace\", [:write])\n    > Extrace.calls({:queue, :in, fn(_) -> :return end}, 3,\n    >                  [{:io_server, dev}])\n    1\n    >\n    Recon tracer rate limit tripped.\n    > File.close(dev).\n\nThe only output still sent to the Group Leader is the rate limit\nbeing tripped, and any errors. The rest will be sent to the other IO\nserver (see http://erlang.org/doc/apps/stdlib/io_protocol.html).","ref":"Extrace.html#module-structure"},{"type":"function","title":"Extrace.calls/2","doc":"Equivalent to `calls/3`.","ref":"Extrace.html#calls/2"},{"type":"function","title":"Extrace.calls/3","doc":"Allows to set trace patterns and pid specifications to trace\nfunction calls.\n\nThe basic calls take the trace patterns as tuples of the form\n`{module, function, args}` where:\n\n - `module` is any Elixir or Erlang module (e.g `Enum` or `:queue`)\n - `function` is any atom representing a function, or the wildcard\n    pattern (`:_`)\n - `args` is either the arity of a function (`0`..`255`), a wildcard\n    pattern (`:_`),\n    a [match specification](http://learnyousomeerlang.com/ets#you-have-been-selected)\n    or a function from a shell session that can be transformed into\n    a match specification\n\nThere is also an argument specifying either a maximal count (a\nnumber) of trace messages to be received, or a maximal frequency\n(`{num, millisecs}`).\n\nHere are examples of things to trace:\n\n- All calls from the `:queue` module, with 10 calls printed at most:\n  `Extrace.calls({:queue, :_, :_}, 10)`\n- All calls to `:lists.seq(a, b)`, with 100 calls printed at most:\n  `Extrace.calls({:lists, :seq, 2}, 100)`\n- All calls to `:lists.seq(a, b)`, with 100 calls per second at most:\n  `Extrace.calls({:lists, :seq, 2}, {100, 1000})`\n- All calls to `:lists.seq(a, b, 2)` (all sequences increasing by two)\n  with 100 calls at most:\n  `Extrace.calls({:lists, :seq, fn([_, _, 2]) -> :ok end}, 100)`\n- All calls to `:erlang.iolist_to_binary/1` made with a binary as an\n  argument already (kind of useless conversion!):\n  `Extrace.calls({:erlang, :iolist_to_binary, fn([x]) when is_binary(x) -> :ok end}, 10)`\n- Calls to the queue module only in a given process `pid`, at a rate\n  of 50 per second at most:\n  `Extrace.calls({:queue, :_, :_}, {50, 1000}, [pid: pid])`\n- Print the traces with the function arity instead of literal\n  arguments:\n  `Extrace.calls(tspec, max, [args: :arity])`\n- Matching the `filter/2` functions of both `dict` and `lists`\n  modules, across new processes only:\n  `Extrace.calls([{:dict, :filter, 2}, {:lists, :filter, 2}], 10, [pid: :new])`\n- Tracing the `handle_call/3` functions of a given module for all\n  new processes, and those of an existing one registered with\n  `gproc`:\n  `Extrace.calls({mod, :handle_call, 3}, {10, 100}, [{:pid, [{:via, :gproc, name}, :new]}`\n- Show the result of a given function call:\n  `Extrace.calls({mod, fun, fn(_) -> :return end}, max, opts)`\n  or\n  `Extrace.calls({mod, fun, [{:_, [], [{:return_trace}]}]}, max, opts)`,\n  the important bit being the `:return` or the `{:return_trace}`\n  match spec value.\n\nThere's a few more combination possible, with multiple trace\npatterns per call, and more options:\n\n- `{:pid, pid_spec}`: which processes to trace. Valid options is any\n  of `all`, `new`, `existing`, or a process descriptor (`{a, b, c}`,\n  `\" \"`, an atom representing a name, `{:global, name}`,\n  `{:via, registrar, name}`, or a pid). It's also possible to specify\n  more than one by putting them in a list.\n- `{:timestamp, :formatter | :trace}`: by default, the formatter\n  process adds timestamps to messages received. If accurate\n  timestamps are required, it's possible to force the usage of\n  timestamps within trace messages by adding the option\n  `{:timestamp, :trace}`.\n- `{:args, :arity | :args}`: whether to print arity in function\n  calls or their (by default) literal representation.\n- `{:scope, :global | :local}`: by default, only `global` (fully\n  qualified function calls) are traced, not calls made internally.\n  To force tracing of local calls, pass in `{:scope, :local}`. This\n  is useful whenever you want to track the changes of code in a\n  process that isn't called with `module.fun(args)`, but just\n  `fun(args)`.\n- `{:formatter, fn(term) -> io_data() end}`: override the default\n   formatting functionality provided by Extrace.\n- `{:io_server, pid() | atom()}`: by default, recon logs to the\n  current group leader, usually the shell. This option allows to\n  redirect trace output to a different IO server (such as a file\n  handle).\n\nAlso note that putting extremely large `max` values (i.e. `99999999`\nor `{10000, 1}`) will probably negate most of the safe-guarding this\nlibrary does and be dangerous to your node. Similarly, tracing\nextremely large amounts of function calls (all of them, or all of\n`:io` for example) can be risky if more trace messages are generated\nthan any process on the node could ever handle, despite the\nprecautions taken by this library.","ref":"Extrace.html#calls/3"},{"type":"function","title":"Extrace.clear/0","doc":"Stops all tracing at once.","ref":"Extrace.html#clear/0"},{"type":"function","title":"Extrace.format/1","doc":"The default trace formatting functionality provided by Extrace.\nThis can be overridden by passing\n`{:formatter, fn(term) -> io_data() end}` as an option to `calls/3`.","ref":"Extrace.html#format/1"},{"type":"function","title":"Extrace.to_erl_tspec/1","doc":"Returns tspec with its `shellfun` replaced with `matchspec`.\nThis function is used by `calls/2` and `calls/3`.","ref":"Extrace.html#to_erl_tspec/1"},{"type":"type","title":"Extrace.args/0","doc":"","ref":"Extrace.html#t:args/0"},{"type":"type","title":"Extrace.f/0","doc":"","ref":"Extrace.html#t:f/0"},{"type":"type","title":"Extrace.formatterfun/0","doc":"","ref":"Extrace.html#t:formatterfun/0"},{"type":"type","title":"Extrace.matchspec/0","doc":"","ref":"Extrace.html#t:matchspec/0"},{"type":"type","title":"Extrace.max/0","doc":"","ref":"Extrace.html#t:max/0"},{"type":"type","title":"Extrace.max_rate/0","doc":"","ref":"Extrace.html#t:max_rate/0"},{"type":"type","title":"Extrace.max_traces/0","doc":"","ref":"Extrace.html#t:max_traces/0"},{"type":"type","title":"Extrace.millisecs/0","doc":"","ref":"Extrace.html#t:millisecs/0"},{"type":"type","title":"Extrace.mod/0","doc":"","ref":"Extrace.html#t:mod/0"},{"type":"type","title":"Extrace.num_matches/0","doc":"","ref":"Extrace.html#t:num_matches/0"},{"type":"type","title":"Extrace.options/0","doc":"","ref":"Extrace.html#t:options/0"},{"type":"type","title":"Extrace.pidspec/0","doc":"","ref":"Extrace.html#t:pidspec/0"},{"type":"type","title":"Extrace.shellfun/0","doc":"","ref":"Extrace.html#t:shellfun/0"},{"type":"type","title":"Extrace.tspec/0","doc":"","ref":"Extrace.html#t:tspec/0"},{"type":"extras","title":"Original License","doc":"Copyright (c) 2015 - 2016, Tatsuya Kawano\nAll rights reserved.\n\nRedistribution and use in source and binary forms, with or without\nmodification, are permitted provided that the following conditions are met:\n\n* Redistributions of source code must retain the above copyright notice, this\n  list of conditions and the following disclaimer.\n\n* Redistributions in binary form must reproduce the above copyright notice,\n  this list of conditions and the following disclaimer in the documentation\n  and/or other materials provided with the distribution.\n\n* Neither the name of recon_ex nor the names of its\n  contributors may be used to endorse or promote products derived from\n  this software without specific prior written permission.\n\nTHIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS \"AS IS\"\nAND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE\nIMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE\nDISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE\nFOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL\nDAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR\nSERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER\nCAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,\nOR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE\nOF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.","ref":"original-license.html"},{"type":"extras","title":"License","doc":"Apache License\nVersion 2.0, January 2004\nhttp://www.apache.org/licenses/\n\nTERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n1. Definitions.\n\n\"License\" shall mean the terms and conditions for use, reproduction,\nand distribution as defined by Sections 1 through 9 of this document.\n\n\"Licensor\" shall mean the copyright owner or entity authorized by\nthe copyright owner that is granting the License.\n\n\"Legal Entity\" shall mean the union of the acting entity and all\nother entities that control, are controlled by, or are under common\ncontrol with that entity. For the purposes of this definition,\n\"control\" means (i) the power, direct or indirect, to cause the\ndirection or management of such entity, whether by contract or\notherwise, or (ii) ownership of fifty percent (50%) or more of the\noutstanding shares, or (iii) beneficial ownership of such entity.\n\n\"You\" (or \"Your\") shall mean an individual or Legal Entity\nexercising permissions granted by this License.\n\n\"Source\" form shall mean the preferred form for making modifications,\nincluding but not limited to software source code, documentation\nsource, and configuration files.\n\n\"Object\" form shall mean any form resulting from mechanical\ntransformation or translation of a Source form, including but\nnot limited to compiled object code, generated documentation,\nand conversions to other media types.\n\n\"Work\" shall mean the work of authorship, whether in Source or\nObject form, made available under the License, as indicated by a\ncopyright notice that is included in or attached to the work\n(an example is provided in the Appendix below).\n\n\"Derivative Works\" shall mean any work, whether in Source or Object\nform, that is based on (or derived from) the Work and for which the\neditorial revisions, annotations, elaborations, or other modifications\nrepresent, as a whole, an original work of authorship. For the purposes\nof this License, Derivative Works shall not include works that remain\nseparable from, or merely link (or bind by name) to the interfaces of,\nthe Work and Derivative Works thereof.\n\n\"Contribution\" shall mean any work of authorship, including\nthe original version of the Work and any modifications or additions\nto that Work or Derivative Works thereof, that is intentionally\nsubmitted to Licensor for inclusion in the Work by the copyright owner\nor by an individual or Legal Entity authorized to submit on behalf of\nthe copyright owner. For the purposes of this definition, \"submitted\"\nmeans any form of electronic, verbal, or written communication sent\nto the Licensor or its representatives, including but not limited to\ncommunication on electronic mailing lists, source code control systems,\nand issue tracking systems that are managed by, or on behalf of, the\nLicensor for the purpose of discussing and improving the Work, but\nexcluding communication that is conspicuously marked or otherwise\ndesignated in writing by the copyright owner as \"Not a Contribution.\"\n\n\"Contributor\" shall mean Licensor and any individual or Legal Entity\non behalf of whom a Contribution has been received by Licensor and\nsubsequently incorporated within the Work.\n\n2. Grant of Copyright License. Subject to the terms and conditions of\nthis License, each Contributor hereby grants to You a perpetual,\nworldwide, non-exclusive, no-charge, royalty-free, irrevocable\ncopyright license to reproduce, prepare Derivative Works of,\npublicly display, publicly perform, sublicense, and distribute the\nWork and such Derivative Works in Source or Object form.\n\n3. Grant of Patent License. Subject to the terms and conditions of\nthis License, each Contributor hereby grants to You a perpetual,\nworldwide, non-exclusive, no-charge, royalty-free, irrevocable\n(except as stated in this section) patent license to make, have made,\nuse, offer to sell, sell, import, and otherwise transfer the Work,\nwhere such license applies only to those patent claims licensable\nby such Contributor that are necessarily infringed by their\nContribution(s) alone or by combination of their Contribution(s)\nwith the Work to which such Contribution(s) was submitted. If You\ninstitute patent litigation against any entity (including a\ncross-claim or counterclaim in a lawsuit) alleging that the Work\nor a Contribution incorporated within the Work constitutes direct\nor contributory patent infringement, then any patent licenses\ngranted to You under this License for that Work shall terminate\nas of the date such litigation is filed.\n\n4. Redistribution. You may reproduce and distribute copies of the\nWork or Derivative Works thereof in any medium, with or without\nmodifications, and in Source or Object form, provided that You\nmeet the following conditions:\n\n(a) You must give any other recipients of the Work or\nDerivative Works a copy of this License; and\n\n(b) You must cause any modified files to carry prominent notices\nstating that You changed the files; and\n\n(c) You must retain, in the Source form of any Derivative Works\nthat You distribute, all copyright, patent, trademark, and\nattribution notices from the Source form of the Work,\nexcluding those notices that do not pertain to any part of\nthe Derivative Works; and\n\n(d) If the Work includes a \"NOTICE\" text file as part of its\ndistribution, then any Derivative Works that You distribute must\ninclude a readable copy of the attribution notices contained\nwithin such NOTICE file, excluding those notices that do not\npertain to any part of the Derivative Works, in at least one\nof the following places: within a NOTICE text file distributed\nas part of the Derivative Works; within the Source form or\ndocumentation, if provided along with the Derivative Works; or,\nwithin a display generated by the Derivative Works, if and\nwherever such third-party notices normally appear. The contents\nof the NOTICE file are for informational purposes only and\ndo not modify the License. You may add Your own attribution\nnotices within Derivative Works that You distribute, alongside\nor as an addendum to the NOTICE text from the Work, provided\nthat such additional attribution notices cannot be construed\nas modifying the License.\n\nYou may add Your own copyright statement to Your modifications and\nmay provide additional or different license terms and conditions\nfor use, reproduction, or distribution of Your modifications, or\nfor any such Derivative Works as a whole, provided Your use,\nreproduction, and distribution of the Work otherwise complies with\nthe conditions stated in this License.\n\n5. Submission of Contributions. Unless You explicitly state otherwise,\nany Contribution intentionally submitted for inclusion in the Work\nby You to the Licensor shall be under the terms and conditions of\nthis License, without any additional terms or conditions.\nNotwithstanding the above, nothing herein shall supersede or modify\nthe terms of any separate license agreement you may have executed\nwith Licensor regarding such Contributions.\n\n6. Trademarks. This License does not grant permission to use the trade\nnames, trademarks, service marks, or product names of the Licensor,\nexcept as required for reasonable and customary use in describing the\norigin of the Work and reproducing the content of the NOTICE file.\n\n7. Disclaimer of Warranty. Unless required by applicable law or\nagreed to in writing, Licensor provides the Work (and each\nContributor provides its Contributions) on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\nimplied, including, without limitation, any warranties or conditions\nof TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\nPARTICULAR PURPOSE. You are solely responsible for determining the\nappropriateness of using or redistributing the Work and assume any\nrisks associated with Your exercise of permissions under this License.\n\n8. Limitation of Liability. In no event and under no legal theory,\nwhether in tort (including negligence), contract, or otherwise,\nunless required by applicable law (such as deliberate and grossly\nnegligent acts) or agreed to in writing, shall any Contributor be\nliable to You for damages, including any direct, indirect, special,\nincidental, or consequential damages of any character arising as a\nresult of this License or out of the use or inability to use the\nWork (including but not limited to damages for loss of goodwill,\nwork stoppage, computer failure or malfunction, or any and all\nother commercial damages or losses), even if such Contributor\nhas been advised of the possibility of such damages.\n\n9. Accepting Warranty or Additional Liability. While redistributing\nthe Work or Derivative Works thereof, You may choose to offer,\nand charge a fee for, acceptance of support, warranty, indemnity,\nor other liability obligations and/or rights consistent with this\nLicense. However, in accepting such obligations, You may act only\non Your own behalf and on Your sole responsibility, not on behalf\nof any other Contributor, and only if You agree to indemnify,\ndefend, and hold each Contributor harmless for any liability\nincurred by, or claims asserted against, such Contributor by reason\nof your accepting any such warranty or additional liability.\n\nEND OF TERMS AND CONDITIONS","ref":"license.html"},{"type":"extras","title":"Overview","doc":"# Extrace\n\n[![Hex.pm Version](https://img.shields.io/hexpm/v/extrace.svg?style=flat-square)](https://hex.pm/packages/extrace)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg?style=flat-square)](https://hexdocs.pm/extrace/)\n[![Total Download](https://img.shields.io/hexpm/dt/extrace.svg?style=flat-square)](https://hex.pm/packages/extrace)\n[![License](https://img.shields.io/hexpm/l/extrace.svg?style=flat-square)](https://github.com/redink/extrace/blob/master/LICENSE.md)\n[![Last Updated](https://img.shields.io/github/last-commit/redink/extrace.svg?style=flat-square)](https://github.com/redink/extrace/commits/master)\n\nExtrace is an Elixir wrapper for [Recon Trace](https://ferd.github.io/recon/recon_trace.html).","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"The package can be installed by adding `:extrace` to your list of dependencies\nin `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:extrace, \"~> 0.3.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Examples - Overview","doc":"Set point:\n\n```elixir\niex> Extrace.calls([{Enum, :take_random, fn _ -> :return end}, {Enum, :count, fn _ -> :return end}], 100, [scope: :local])\n4\n```\n\nNote that the functions to be traced (`:take_random` and `:count` in the example above) can only be private if `scope: :local` is set.\n\nOne function executed:\n\n```elixir\niex(2)> Enum.take_random([1,2,3,4], 200)\n[2, 4, 3, 1]\n\n18:42:10.834670 <0.183.0> Enum.take_random([1, 2, 3, 4], 200)\n\n18:42:10.840212 <0.183.0> Enum.take_random(%{0 => 2, 1 => 4, 2 => 3, 3 => 1}, 4, [])\n\n18:42:10.840949 <0.183.0> Enum.take_random(%{0 => 2, 1 => 4, 2 => 3, 3 => 1}, 3, [1])\n\n18:42:10.841064 <0.183.0> Enum.take_random(%{0 => 2, 1 => 4, 2 => 3, 3 => 1}, 2, [3, 1])\n\n18:42:10.841202 <0.183.0> Enum.take_random(%{0 => 2, 1 => 4, 2 => 3, 3 => 1}, 1, [4, 3, 1])\n\n18:42:10.841330 <0.183.0> Enum.take_random(%{0 => 2, 1 => 4, 2 => 3, 3 => 1}, 0, [2, 4, 3, 1])\n\n18:42:10.841440 <0.183.0> Enum.take_random/3 --> [2, 4, 3, 1]\n\n18:42:10.841521 <0.183.0> Enum.take_random/3 --> [2, 4, 3, 1]\n\n18:42:10.841606 <0.183.0> Enum.take_random/3 --> [2, 4, 3, 1]\n\n18:42:10.841685 <0.183.0> Enum.take_random/3 --> [2, 4, 3, 1]\n\n18:42:10.841959 <0.183.0> Enum.take_random/3 --> [2, 4, 3, 1]\n\n18:42:10.842046 <0.183.0> Enum.take_random/2 --> [2, 4, 3, 1]\n```\n\nAnother function executed:\n\n```elixir\niex(4)> Enum.count([1,2,3,4])\n4\n\n18:42:27.383667 <0.183.0> Enum.count([1, 2, 3, 4])\n\n18:42:27.383795 <0.183.0> Enum.count/1 --> 4\n```","ref":"readme.html#examples"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2015, 2016 Tatsuya Kawano under [MIT License](./ORIGINAL-LICENSE.md). Fork from [https://github.com/tatsuya6502/recon_ex](https://github.com/tatsuya6502/recon_ex)\n\nCopyright (c) 2019 redink\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\nhttp://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"readme.html#copyright-and-license"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}